<?php

namespace App\Controller;

use App\Entity\Snip;
use App\Entity\SnipContent;
use App\Security\Voter\SnipVoter;
use App\Service\SnipContent\FlowChartTreeBuilder;
use App\Service\SnipContent\SnipContentService;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;

#[Route('/version/{snip}', name: 'version')]
class VersionController extends AbstractController
{
    public function __construct(
        private readonly SnipContentService $contentService,
    ) {}

    #[Route('/', name: '_index')]
    public function index(Snip $snip, FlowChartTreeBuilder $builder): Response
    {
        $this->denyAccessUnlessGranted(SnipVoter::EDIT, $snip);

        $buildTree = $builder->buildTree($snip);
        return $this->render('version/index.html.twig', [
            'snip' => $snip,
//            'versions' => new GitTreeBuilder($snip)->buildTree($snip->getSnipContents()->first()),
            'versions' => $buildTree,
        ]);
    }

    #[Route('/set/{version}', name: '_set')]
    public function set(Snip $snip, SnipContent $version): Response
    {
        $this->denyAccessUnlessGranted(SnipVoter::EDIT, $snip);

        $this->contentService->setVersion($snip, $version);
        $this->addFlash('success', 'Snip version set to ' . $version->getId());
        return $this->redirectToRoute('snip_single', ['snip' => $snip->getId()]);
    }
}