<?php

namespace App\Service\SnipContent;

use App\Entity\Snip;
use App\Entity\SnipContent;

class GitTreeBuilder
{
    private string $activeBranch = 'main';

    public function __construct(private readonly Snip $snip) {}

    public function buildTree(SnipContent $content, string $branch = 'main'): array
    {
        $tree = [];
        if ($this->activeBranch !== $branch) {
            $tree[] = $this->checkout($branch);
        }
        $commit = sprintf('commit id:"%s"', $content);
        if ($this->snip->getActiveVersion() === $content) {
            $commit .= ' tag: "active" type: REVERSE';
        }
        $tree[] = $commit;

        $first = true;
        foreach ($content->getChildren() as $child) {
            if (!$first) {
                $tree[] = $this->branch($child);
                $tree[] = $this->checkout($branch);
            }
            $first = false;
        }

        $first = true;
        foreach ($content->getChildren() as $child) {
            if ($first) {
                $myBranch = $branch;
            } else {
                $tree[] = $this->checkout($child);
                $myBranch = $child;
            }
            $tree = array_merge($tree, self::buildTree($child, $myBranch));
            $first = false;
        }

        return $tree;
    }

    private function branch(string $branch): string
    {
        $this->activeBranch = $branch;
        return sprintf('branch %s', $branch);
    }

    private function checkout(string $branch): string
    {
        $this->activeBranch = $branch;
        return sprintf('checkout %s', $branch);
    }
}